/*
  Arduino Incubator [INQ]

  Displays and controls the internal temperature and humidity of an incubator.
  More information can be found on www.instructables.com/member/lightbluecrab
  
  created 23 March 2019
  
  by @lightbluecrab

 */

#include <Wire.h>   //https://github.com/esp8266/Arduino/tree/master/libraries/Wire
#include <LiquidCrystal_I2C.h>    //https://github.com/fdebrabander/Arduino-LiquidCrystal-I2C-library
#include <DHT.h>    //https://github.com/adafruit/DHT-sensor-library

int desiredHumidity = 85;   //ideal relative humidity percentage
int desiredHumidityTolerance = 1;   //humidity percentage tolerance for the fan to turn on

int measuringInterval = 5;   //measuring interval (in seconds)

int ventilationInterval = 30;   //ventilation interval (in minutes)
int ventilationDuration = 5;    //ventilation time (in seconds)

const float adjustmentRange[2] = {36.0, 38.0};    //potentiometer adjustment range (min temp, max temp)

const int transistorFanPin = 10;   //transistor base pin
const int relayLightPin = 8;   //relay signal pin
const int potentiometerPin = A3;    //poteniometer Vout
#define DHTPIN 2    //DHTxx signal pin
#define DHTTYPE DHT22   //DHT type (DHT11/DHT22)


unsigned long previousMeasuringMillis = 0;   //stores the last time checked the DHT values (in ms)
unsigned long previousVentilationMillis = 0;    //stores last active ventilation time (in ms)

float temp;
float hum;
bool lightState;   //state of relay
bool fanState;   //state of transistor

LiquidCrystal_I2C lcd(0x27, 16, 2);   //define specifications (I2C adress, chars, rows)
DHT dht(DHTPIN,DHTTYPE);

//function for mapping floats
float mapfloat(float x, float in_min, float in_max, float out_min, float out_max)
{
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}

//lcd chars and animations for indicating light and fan state
uint8_t lightbulbOffIcon[8] = {0x00,0x0A,0x0A,0x11,0x11,0x11,0x0E,0x00};
uint8_t lightbulbOnIcon[8] = {0x00,0x0A,0x0A,0x1F,0x1F,0x1F,0x0E,0x00};

void fanOffIcon() {
  lcd.setCursor(10, 0);
  lcd.print("/");
}
void fanOnIcon() {   
  for (int i = 1; i <= 3; i++) {
    lcd.setCursor(10, 0);
    switch(i) {
      case 1: lcd.print("/");
        break;
      case 2: lcd.print("-");
        break;
      case 3: lcd.print("|");
        break;
    }
    delay(200);
  }
}



void setup() {
  pinMode(transistorFanPin, OUTPUT);    //set output pins
  pinMode(relayLightPin, OUTPUT);

  digitalWrite(relayLightPin, HIGH);    //turn on light
  lightState = true;

  lcd.begin();    //intialize lcd
  lcd.backlight();

  lcd.createChar(0, lightbulbOffIcon);    //creating chars for displaying light state
  lcd.createChar(1, lightbulbOnIcon);
  
  dht.begin();    //initialize DHT
}


void loop() { 

  float desiredTemperature = mapfloat(analogRead(potentiometerPin), 0.0, 1023.0, adjustmentRange[0], adjustmentRange[1]);   //reads the current potentiometer value

  //lcd row 1 output
  lcd.setCursor(0, 0);
  lcd.print("T: "); lcd.print(temp, 1); lcd.print((char)223); lcd.print("C|"); (fanState) ? fanOnIcon() : fanOffIcon(); lcd.setCursor(11, 0); lcd.print("|"); lcd.print(" DT:");

  //lcd row 2 output
  lcd.setCursor(0, 1);
  lcd.print("H: "); lcd.print(hum, 1); lcd.print("% |"); (lightState) ? lcd.write(1) : lcd.write(0); lcd.setCursor(11, 1); lcd.print("|"); lcd.print(desiredTemperature);


  //measuring loop (according to the set interval)
  if (millis() - previousMeasuringMillis >= (measuringInterval * 1000)) {
      previousMeasuringMillis = millis();
      //reads temperature and humidity
      temp = dht.readTemperature();
      hum = dht.readHumidity();
      delay(20);

      //check temperature and humidity and possibly change it through fan/light control
      if (temp > desiredTemperature) {
         //checks whether temperature needs to be lowered with humidity loss
        if (hum > desiredHumidity) {
         digitalWrite(transistorFanPin, HIGH);   //turns on fan
         fanState = true;
        }
         //checks whether temperature needs to be lowered without humidity loss
        if (hum < desiredHumidity) {
         digitalWrite(relayLightPin, LOW);   //turns off light
         lightState = false;
         digitalWrite(transistorFanPin, LOW);   //turns off fan
         fanState = false;
        }
    }
   else {   //normal state (fan off and light on)
    digitalWrite(relayLightPin, HIGH);
    lightState = true;

    //turns on the fan if the humidity is to high
    if (hum > (desiredHumidity + desiredHumidityTolerance) && !fanState) {
      digitalWrite(transistorFanPin, HIGH);
      fanState = true;
    }
    //turns off fan if the humidity is to low
    if ((hum < desiredHumidity) && fanState) {
      digitalWrite(transistorFanPin, LOW);
      fanState = false;
      }
    
    }
      
  }


  //ventilation loop
  if ((millis() - previousVentilationMillis >= (ventilationInterval * 60000)) && !fanState) {
    digitalWrite(transistorFanPin, HIGH);   //turns on fan
    fanState = true;
  }

  if (millis() - previousVentilationMillis >= ((ventilationInterval * 60000) + (ventilationDuration * 1000))) {
    previousVentilationMillis = millis();   //reset delay
    digitalWrite(transistorFanPin, LOW);   //turns on fan
    fanState = false;
  }
  
  
  delay(100);
}
